@@ -1,5 +1,10 @@
-from __future__ import print_function
-
+import cftime
+from cftime import datetime as datetimex
+from cftime import real_datetime
+from cftime import (Datetime360Day, DatetimeAllLeap,
+                    DatetimeGregorian, DatetimeJulian, DatetimeNoLeap,
+                    DatetimeProlepticGregorian, _parse_date,
+                    date2index, date2num, num2date,  UNIT_CONVERSION_FACTORS)
 import copy
 import operator
 import sys
@@ -12,14 +17,6 @@ import numpy as np
 import pytest
 from numpy.testing import assert_almost_equal, assert_equal
 
-import cftime
-from cftime import datetime as datetimex
-from cftime import real_datetime
-from cftime import (Datetime360Day, DatetimeAllLeap,
-                    DatetimeGregorian, DatetimeJulian, DatetimeNoLeap,
-                    DatetimeProlepticGregorian, _parse_date,
-                    date2index, date2num, num2date,  UNIT_CONVERSION_FACTORS)
-
 try:
     from datetime import timezone
 except ImportError: # python2.7
@@ -76,7 +73,7 @@ def adjust_calendar(calendar):
     # check for and remove calendar synonyms.
     calendar = calendar.lower()
     if calendar == 'gregorian' or calendar == 'standard':
-        return 'gregorian'
+        return 'standard'
     elif calendar == 'noleap' or calendar == '365_day':
         return 'noleap'
     elif calendar == 'all_leap' or calendar == '366_day':
@@ -509,7 +506,7 @@ class cftimeTestCase(unittest.TestCase):
         units = 'days since 0001-01-01'
         for cap_cal, low_cal in (('STANDARD', 'standard'),
                                  ('NoLeap', 'noleap'),
-                                 ('Gregorian', 'gregorian'),
+                                 ('Standard', 'standard'),
                                  ('ALL_LEAP', 'all_leap')):
             d1 = date2num(d, units, cap_cal)
             d2 = date2num(d, units, low_cal)
@@ -558,15 +555,13 @@ class cftimeTestCase(unittest.TestCase):
         # n should always be 0 as all units refer to the same point in time
         assert_almost_equal(n, 0)
 
-        # list around missing dates in Gregorian calendar
+        # list around missing dates in mixed Julian/Gregorian calendar
         # scalar
         units = 'days since 0001-01-01 12:00:00'
-        t1 = date2num(datetime(1582, 10, 4), units, calendar='gregorian')
-        t2 = date2num(datetime(1582, 10, 15), units, calendar='gregorian')
+        t1 = date2num(datetime(1582, 10, 4), units, calendar='standard')
+        t2 = date2num(datetime(1582, 10, 15), units, calendar='standard')
         self.assertEqual(t1+1, t2)
         # list
-        t1, t2 = date2num([datetime(1582, 10, 4), datetime(1582, 10, 15)], units, calendar='gregorian')
-        self.assertEqual(t1+1, t2)
         t1, t2 = date2num([datetime(1582, 10, 4), datetime(1582, 10, 15)], units, calendar='standard')
         self.assertEqual(t1+1, t2)
         # this should fail: days missing in Gregorian calendar
@@ -863,7 +858,7 @@ class cftimeTestCase(unittest.TestCase):
         jdref=2400000
         with warnings.catch_warnings():
             warnings.simplefilter("ignore",category=cftime.CFWarning)
-            for calendar in ['julian','gregorian','proleptic_gregorian']:
+            for calendar in ['julian','standard','proleptic_gregorian']:
                 has_year_zero=False
                 try:
                     # this should raise ValueError
@@ -919,6 +914,13 @@ class cftimeTestCase(unittest.TestCase):
         assert(d.has_year_zero==True)
         d = d.replace(year=0)
         assert(d.has_year_zero==True)
+        # test leap year function
+        assert(cftime.is_leap_year(2000,calendar='standard'))
+        assert(cftime.is_leap_year(-1,calendar='standard'))
+        assert(cftime.is_leap_year(0,calendar='standard',has_year_zero=True))
+        assert(not cftime.is_leap_year(1,calendar='standard',has_year_zero=True))
+        assert(not cftime.is_leap_year(1,calendar='365_day'))
+        assert(cftime.is_leap_year(1,calendar='366_day'))
 
 
 class TestDate2index(unittest.TestCase):
@@ -959,6 +961,10 @@ class TestDate2index(unittest.TestCase):
         self.standardtime = self.TestTime(datetime(1950, 1, 1), 366, 24,
                                           'hours since 1900-01-01', 'standard')
 
+        self.issue272time = self.TestTime(datetime(1950, 1, 1), 5, 24,
+                                          'hours since 1900-01-01', 'standard')
+        self.issue272time._data=np.array([1053144, 1053150, 1053156, 1053157,
+            1053162],np.int32)
         self.time_vars = {}
         self.time_vars['time'] = CFTimeVariable(
             values=self.standardtime,
@@ -1115,6 +1121,18 @@ class TestDate2index(unittest.TestCase):
                            select='nearest')
         assert(index == 11)
 
+    def test_issue272(self):
+        timeArray = self.issue272time
+        date = datetime(2020, 2, 22, 13)
+        assert(date2index(date, timeArray, calendar="gregorian",
+            select="exact")==3)
+        assert(date2index(date, timeArray, calendar="gregorian",
+            select="before")==2)
+        assert(date2index(date, timeArray, calendar="gregorian",
+            select="after")==4)
+        assert(date2index(date, timeArray, calendar="gregorian",
+            select="nearest")==3)
+
 
 class issue584TestCase(unittest.TestCase):
     """Regression tests for issue #584."""
@@ -1655,7 +1673,7 @@ def test_num2date_only_use_cftime_datetimes_post_gregorian(
 
 
 def test_repr():
-    expected = "cftime.datetime(2000, 1, 1, 0, 0, 0, 0, calendar='gregorian', has_year_zero=False)"
+    expected = "cftime.datetime(2000, 1, 1, 0, 0, 0, 0, calendar='standard', has_year_zero=False)"
     assert repr(datetimex(2000, 1, 1, calendar='standard')) == expected
     expected = "cftime.datetime(2000, 1, 1, 0, 0, 0, 0, calendar='', has_year_zero=False)"
     assert repr(datetimex(2000, 1, 1, calendar=None)) == expected
@@ -2035,6 +2053,53 @@ def test_date2num_num2date_roundtrip(encoding_units, freq, calendar):
         meets_tolerance = np.abs(decoded - times) <= tolerance
         assert np.all(meets_tolerance)
 
+def test_date2num_missing_data():
+    # Masked array
+    a = [
+        cftime.DatetimeGregorian(2000, 12, 1),
+        cftime.DatetimeGregorian(2000, 12, 2),
+        cftime.DatetimeGregorian(2000, 12, 3),
+        cftime.DatetimeGregorian(2000, 12, 4),
+    ]
+    mask = [True, False, True, False]
+    array = np.ma.array(a, mask=mask)
+    out = date2num(array, units="days since 2000-12-01", calendar="standard")
+    assert ((out == np.ma.array([-99, 1, -99, 3] , mask=mask)).all())
+    assert ((out.mask == mask).all())
+
+    # Scalar masked array
+    a = cftime.DatetimeGregorian(2000, 12, 1)
+    mask = True
+    array = np.ma.array(a, mask=mask)
+    out = date2num(array, units="days since 2000-12-01", calendar="standard")
+    assert out is np.ma.masked
+
+
+def test_num2date_preserves_shape():
+    # The optimized num2date algorithm operates on a flattened array.  This
+    # check ensures that the original shape of the times is restored in the 
+    # result.
+    a = np.array([[0, 1, 2], [3, 4, 5]])
+    result = num2date(a, units="days since 2000-01-01", calendar="standard")
+    expected = np.array([cftime.DatetimeGregorian(2000, 1, i) for i in range(1, 7)]).reshape((2, 3))
+    np.testing.assert_equal(result, expected)
+
+
+def test_num2date_preserves_order():
+    # The optimized num2date algorithm sorts the encoded times before decoding them.
+    # This check ensures that the order of the times is restored in the result.
+    a = np.array([1, 0])
+    result = num2date(a, units="days since 2000-01-01", calendar="standard")
+    expected = np.array([cftime.DatetimeGregorian(2000, 1, i) for i in [2, 1]])
+    np.testing.assert_equal(result, expected)
+
+
+def test_num2date_empty_array():
+    a = np.array([[]])
+    result = num2date(a, units="days since 2000-01-01", calendar="standard")
+    expected = np.array([[]], dtype="O")
+    np.testing.assert_equal(result, expected)
+
 
 if __name__ == '__main__':
     unittest.main()
